####  R script for simulations of population colonization in 
# "Understanding population structure in the evolutionary context: population-specific FST and pairwise FST" 
# coded by Reiichiro Nakamichi
# revison 3, April 18, 2021


## CRAN packages 
library(ape)                    # Computing neibour joining (NJ) tree
library(FinePop2)               # Computing FST values 

##################################################
# Parameter setting
map.file <- "map1way.txt"       # Specifying a map file for coronation scenarios(map1way, map2way, map3way,mapSS_edge or mapSS_center)
num.gen <- 260                  # Number of generations simulated for map1way, map2way, map3way (num.gen should be an integral multiple of interval.mig) 
#num.gen <- 100                 # Number of generations for mapSS_edge and/or mapSS_center
baseNe <- 1e4                   # Effective population size (Ne) of each derived population
scale.ancestor <- 1             # The scale for Ne of the ancestral population
interval.mig <- 10              # The interval of generations for individuals move into the adjacent vacant habitat
num.loci.ancestor <- 10000      # Number of SNPs in the ancestral population
num.loci.new <- 500             # Number of newly derived SNP by mutation during num.gen
init.af.new <- 0.01             # Initial allele frequency of a newly derived SNP
num.ind <- 50                   # Sample sizes (numbers of individuals) from each population after a simulation
gpfile <- "simmig_gp.txt"       # Name of the Genepop file of genotypes obtained from a simulation

##################################################

# Genetic drift of allele frequencies at the neutral SNP loci
makeP.neutral <- function(obsP, Ne){
  cnloci <- dim(obsP)[2]
  obsP2 <- array(NA, dim(obsP))
  colnames(obsP2) <- colnames(obsP)
  for(cpop in 1:npop){
    cNe <- Ne[cpop]
    cP <- obsP[cpop,]
    cP2 <- rbinom(cnloci,cNe,prob=cP)/cNe
    obsP2[cpop,] <- cP2
  }
  return(obsP2)
}

# Generating genotypes of nbirth individuals from allele frequencies (obsP)
makeG <- function(obsP, nbirth){
  cnloci <- dim(obsP)[2]
  cnpop <- dim(obsP)[1]
  obsG <- array(NA, c(npop, nbirth, cnloci))
  for(cpop in 1:cnpop){
    cP <- obsP[cpop,]
    cP <- cbind(1-cP, cP)
    cG <- apply(cP, 1, function(x){
           if(is.na(x[1])){ rep(NA,nbirth*2) }else{ sample(0:1, size=nbirth*2, rep=T,prob=x) }
         })
    cG <- cG[c(T,F),]+cG[c(F,T),]
    obsG[cpop,,] <- cG
  }
  return(obsG)
}

# Generating allele frequencies of new populations after colonization 
migratePxV <- function(obsP, Ne, mig.rate, pop.exist, vacant){ # colonization to vacant habitat
  cnloci <- dim(obsP)[2]
  cnpop <- dim(obsP)[1]
  cmig <- array(NA, dim(connect))
  cmig[,1] <- !is.na(match(connect[,1],which(pop.exist)))
  cmig[,2] <- !is.na(match(connect[,2],which(pop.exist)))
  cmig <- which(if(vacant){cmig[,1]|cmig[,2]}else{cmig[,1]&cmig[,2]})
  cNe <- Ne * pop.exist
  for(cmigi in cmig){
    cpop1 <- as.integer(connect[cmigi, 1])
    cpop2 <- as.integer(connect[cmigi, 2])
    cP1 <- obsP[cpop1,]
    cP2 <- obsP[cpop2,]
    cP1[is.na(cP1)] <- 0
    cP2[is.na(cP2)] <- 0
    cmr <- as.numeric(mig.rate[cmigi])
    cN1 <- cNe[cpop1]
    cN2 <- cNe[cpop2]
    n12 <- as.integer(cN1 * cmr); n1 <- cN1 - n12
    n21 <- as.integer(cN2 * cmr); n2 <- cN2-n21
    cP1m <- (rbinom(cnloci, n1, cP1) + rbinom(cnloci, n21, cP2)) / (n1 + n21)
    cP2m <- (rbinom(cnloci, n2, cP2) + rbinom(cnloci, n12, cP1)) / (n2 + n12)
    cP1m[is.nan(cP1m)] <- NA
    cP2m[is.nan(cP2m)] <- NA
    obsP[cpop1,] <- cP1m
    obsP[cpop2,] <- cP2m
  }
  return(obsP)
}

# Creating newly derived SNPs by mutation
mutateP <- function(npop, init.af, nloci.1step, Ne.scale, pop.exist, generation){
  nL1 <- nloci.1step * Ne.scale
  nL.all <- sum(nL1[pop.exist])
  output.newP <- array(NA,c(npop,nL.all))
  newloci.name <- rep("", nL.all)
  cloc <- 1
  for(cpop in which(pop.exist)){        # nL1 SNPs derived in each population
    cnL1 <- nL1[cpop]
    newP <- array(NA,c(npop,cnL1))
    newP[pop.exist,] <- 0               # Creating SNPs of MAF=0 in all populations
    newP[cpop,] <- init.af[cpop]        # Set MAF=init.af in the derived SNP
    output.newP[,cloc+(1:cnL1)-1] <- newP
    newloci.name[cloc+(1:cnL1)-1] <- paste0("g", generation, "p", cpop, "s",1:cnL1)
    cloc <- cloc + cnL1
  }
  colnames(output.newP) <- newloci.name
  return(output.newP)
}

# Deleting SNPs from the simulation those have very minor allele frequencies lower than the threshold
af.cut.off <- function(obsP, af.threshold){
  meanP <- colMeans(obsP,na.rm=T)
  obsP <- obsP[,meanP>af.threshold,drop=F]
  return(obsP)
}

### SNP simulation ###
map <- as.matrix(read.table(map.file))   
pops <- sort(unique(as.integer(map[,c(1,2)])))
npop <- length(pops)
connect <- map[,c(1,2)]
mig.rate <- map[,3]

Ne <- rep(baseNe, npop)                 # Effective population size of colonized population
Ne[1] <- baseNe * scale.ancestor
Ne.scale <- Ne/min(Ne)

ngen.step <- interval.mig
nstep  <- as.integer(num.gen / interval.mig)

### migration
init.af <- init.af.new
nloci.1step.N <- as.integer((num.loci.new/init.af*2) / (num.gen*sum(Ne.scale)/2)) + 3

# Alelle frequencies of the ancestral populaiton
nloci.ancestor <- num.loci.ancestor * 10
q1 <- 1/Ne[1]
q2 <- 1-1/Ne[1]
C <- 1/log((q2*(1-q1))/(q1*(1-q2)))
d <- log(q1/(1-q1))
u <- runif(nloci.ancestor)
dist.af.ancestor <- 1/(1+exp(-(u/C+d)))
rm(q1,q2,C,d,u);gc()
obsPa <- array(NA, c(npop, nloci.ancestor))
obsPa[1,] <- apply(cbind(dist.af.ancestor,1-dist.af.ancestor), 1, min)

# Performing simulations 
obsPmN <- NULL # SNP: mutaion, neutral
cutoff.0 <- init.af * 1e-3
for(cstep in 1:nstep){
  message(rep("\b",10),cstep,"/",nstep,appendLF=F);flush.console()
  pop.exist <- !is.na(obsPa[,1])
  for(cgen in 1:ngen.step){
    cgen.total <- (cstep-1)*ngen.step+cgen
    # newly derived SNPs 
    obsPmN <- cbind(obsPmN, mutateP(npop, init.af, nloci.1step.N, Ne.scale, pop.exist, cgen.total))
    # migration of individuals between adjacent populations 
    obsPa <- migratePxV(obsPa, Ne, mig.rate, pop.exist, vacant=F)
    obsPmN <- migratePxV(obsPmN, Ne, mig.rate, pop.exist, vacant=F)
    # Genetic drift of allele frequencies at neutral SNPs 
    obsPa <- makeP.neutral(obsPa, Ne)
    obsPmN <- makeP.neutral(obsPmN, Ne)
  }
  # colonization to adjacent vacant habitat
  obsPa <- migratePxV(obsPa, Ne, mig.rate, pop.exist, vacant=T)
  obsPmN <- migratePxV(obsPmN, Ne, mig.rate, pop.exist, vacant=T)
  # Deleting SNPs from the simulation those have very minor allele frequencies lower than the threshold
  obsPa <- af.cut.off(obsPa, af.threshold=cutoff.0)
  obsPmN <- af.cut.off(obsPmN, af.threshold=cutoff.0)
}

cutoff.keep <- 0.001
obsPa.cut <- af.cut.off(obsPa, af.threshold=cutoff.keep)
obsPmN.cut <- af.cut.off(obsPmN, af.threshold=cutoff.keep)

colnames(obsPa.cut) <- paste0("A",1:ncol(obsPa.cut))
colnames(obsPmN.cut) <- paste0("N",colnames(obsPmN.cut))


# Output genotypes created by the simulation to GENEPOP format in a text file
nNs <- num.loci.new
nAs <- num.loci.ancestor
selectP <- function(obsP,n.select){obsP[,sort(sample(1:ncol(obsP),n.select))]}
obsP <- cbind(selectP(obsPmN.cut,nNs), selectP(obsPa.cut,nAs))

nloci <- ncol(obsP)
nind.gp <- num.ind
obsG.gp <- makeG(obsP, nind.gp)

outtxt <- paste0("\"pop=", npop, " snp=", nloci, "\"") 
snpnames <- colnames(obsP)
outtxt <- c(outtxt, paste0(snpnames, collapse="\n"))
for(cpop in 1:npop){
  cgt <- obsG.gp[cpop,,]
  cgt <- apply(cgt, 2, function(x){
    x <- gsub("0","A",x)
    x <- gsub("1","H",x)
    x <- gsub("2","B",x)
    x <- gsub("A","0101",x)
    x <- gsub("H","0102",x)
    x <- gsub("B","0202",x)
  })
  cgt <- apply(cgt, 1, paste0, collapse="\t")
  cind <- paste0("p",cpop,"i",1:nind.gp)
  cgt <- paste0(cind, ", ", cgt)
  cgt <- c("POP",cgt)
  outtxt <- c(outtxt,cgt)
}
cat(outtxt, file=gpfile, sep="\n")
#rm(obsG.gp);gc()


######################################################################
# Data analysis using FinePop2 and ape

popdata <- read.GENEPOP(gpfile) 

n.loci <- popdata$num_loci
loci.name <- popdata$loci_names
n.pop <- popdata$num_pop

# Calculation of population specific FST values
psFST.res <- pop_specificFST(popdata, cov=F)
psFST <- psFST.res$fst[,"FST"]
names(psFST) <- 1:npop
x11()
par(mar=c(4, 4, 1, 0), oma=c(1, 1, 1, 0.5), mgp=c(2, 0.7, 0))
barplot(psFST, names=names(psFST),cex.axis=1.2,cex.lab=1.2,cex=1.2,las=3)
mtext(side=2,line=3,text="Population-specific FST",cex=1.2)
mtext(side=1,line=3.5,text="Population",cex=1.2)

# Calculation of pairwise FST values
pwFST <- pop_pairwiseFST(popdata)
pwFST.mds <- cmdscale(pwFST, k=5)
cmdscale(pwFST, k=1, eig=T)$GOF
cmdscale(pwFST, k=2, eig=T)$GOF
psFST_st <- (psFST-min(psFST))/(max(psFST)-min(psFST))
col.pop <- rgb(1-psFST_st,0,psFST_st, alpha=1)
col.pop0 <- rgb(1-psFST_st,0,psFST_st, alpha=0.5)

x11()
par(mar=c(3, 4, 1, 1), oma=c(1, 1, 1, 0.5), mgp=c(2.2, 0.7, 0))
plot(pwFST.mds[,c(1,2)],type="n",main="",cex.axis=1.5,cex.lab=1.5,cex=1.5,
xlab="mds1 (57%)", ylab="mds2 (39%)")
text(pwFST.mds[,c(1,2)],as.character(1:npop),cex=2,col=col.pop)

# Neighbor-joining unrooted tree based on pairwise FST distance matrix overlaid with population-specific FST values
nj.res <- nj(as.dist(pwFST))
x11()
par(mar=c(0.1,0.1,0.1,0.1))
plot(nj.res, "u", show.tip.label=F)
size.pop <- (popdata$He * 10)^3.5
tiplabels(pch=19, col=col.pop, frame="none", cex=2)#,cex=size.pop)
tiplabels(1:n.pop, col="black", frame="none", cex=1.5,pos=3)
add.scale.bar(x=0.012, y=0,length = NULL, ask = FALSE, lwd = 1, lcol = "black",cex=1.5)

x11();par(mar=c(0.1,0.1,0.1,0.1))
plot(nj.res, "u", show.tip.label=F)
size.pop <- (popdata$He * 10)^2
tiplabels(1:npop, col=col.pop, frame="none", cex=1.5)
#tiplabels(pch=19, col=col.pop, frame="none", cex=size.pop)














